//****************************************************
//******************** Libraries *********************
//****************************************************

#include <SoftwareSerial.h>
#include <PS2Keyboard.h>

PS2Keyboard keyboard;
#define txPin 2 //LCD RX on pin 2 Arduino MEGA
SoftwareSerial LCD = SoftwareSerial(0, txPin); 

//****************************************************
//******************** Constants *********************
//****************************************************

const int IRQpin =  3; //CLOCK blue wire
const int DataPin = 4; //DATA white wire
const int LCDdelay = 2; //general delay for LCD


//****************************************************
//**************** Global variables ******************
//****************************************************

int i=0; //control variable for IF loop
int j; //control variable for FOR loop
int k; //control variable for FOR loop
char input[500];



void setup() 
{ 
  keyboard.begin(DataPin, IRQpin);
  pinMode(txPin, OUTPUT);
  LCD.begin(9600);
  backlightOn();
  cursorOff(); 
  clearLCD();
  lcdPosition(0,0); 
  LCD.print("BOOTING...");
  delay(1000);
  clearLCD();
  lcdPosition(0,0); 
  LCD.print("READY");
  delay(1000);
  clearLCD();
  cursorPos(1,0);
  Serial.begin(9600);
}

void loop() 
{
  if (keyboard.available()) 
  {
    char c = keyboard.read();
    if (c == PS2_ENTER) //by pressing ENTER command will be executed
    {
      cursorOff();
      clearLCD();
      cursorPos(1,0);
      for (int h=0; h<i; h++)
      {
        Serial.print(input[h]);
      }
      Serial.println(" ");
      memset(input, 0, sizeof(input));
      i=0;
    }
    else if (c == PS2_DELETE)
    {
     if (i>0)
    {
      if (i<16)
      {
        lcdPosition(1,i-1); 
        LCD.print(" ");
        cursorPos(1,i-1);
      }
        else
        {
          lcdPosition(1,i-1); 
          LCD.print(" ");
          cursorPos(1,i-1);
          scroll_right();
        }
        i=i-1; 
        }
        }    
    else 
    {
    if (i<38)
    {
    c=toupper(c); 
    input[i]=c;
    if (i>14)
    {
      scroll_left();
      cursorPos(1,15); 
    }
    cursorPos(1,i+1);
    lcdPosition(1,i); 
    LCD.print(c);
    i++;
    }
   }
  }
}



//****************************************************
//************** Function declarations ***************
//****************************************************

//------------------ LCD functions -------------------
  
void lcdPosition(int row, int col) 
{
  LCD.write(0xFE);   
  LCD.write((col + row*64 + 128));   
  delay(LCDdelay);
}
void clearLCD() 
{
  LCD.write(0xFE);   
  LCD.write(0x01);   
  delay(LCDdelay);
}
void backlightOn() 
{  
  LCD.write(0x7C);   
  LCD.write(157);   
  delay(LCDdelay);
}
void backlightOff() 
{  
  LCD.write(0x7C);   
  LCD.write(128);     
  delay(LCDdelay);
}
void serCommand() 
{     
  LCD.write(0xFE);
}  
void cursorPos(int row, int col) //move blinking box cursor to desired position 
{  
  LCD.write(0xFE);  
  LCD.write((col + row*64 + 128)); 
  LCD.write(0xFE);
  LCD.write(0x0D);  
  delay(LCDdelay);
}
void cursorOff() //switch blinking box cursor off 
{ 
  LCD.write(0xFE);
  LCD.write(0x0C);
  delay(LCDdelay);
}  
void scroll_left() //scroll the rows one position to the left
{
  LCD.write(0xFE);
  LCD.write(0x18);
}
void scroll_right() //scroll the rows one position to the right
{
  LCD.write(0xFE);
  LCD.write(0x1C);
}
